#include <stdio.h>
#include "JFileSuckObj.h"
#include "JInteger.h"
#include "JIntegerData.h"
#include "JStringData.h"
#include "JIntegerProperty.h"
#include "JIntegerListProperty.h"
#include "JStringProperty.h"
#include "JLinkObj.h"
#include "JBytesData.h"


char* theJFileSuckObj = JFileSuckObj().Register();


const char* JFileSuckObj::className() const { return "JFileSuckObj";}


JObject* JFileSuckObj::clone() const { return new JFileSuckObj(*this);}


// class constructor
JFileSuckObj::JFileSuckObj() {
  igm = 1 + 1 + 1;
  ogm = 1 + 1;
  enabled = 0;
  file_is_open = 0;
  fp = NULL;
  lastLine = NULL;
}

void JFileSuckObj::writeContent(JOutputStream& os) {
  JAddObj::writeContent(os);
  putObject(os, "fileName", filename);
}

void JFileSuckObj::readContent(JDictionary& dict) {
  JAddObj::readContent(dict);
  JObject* obj_format = getObject( dict, "fileName");
  if ( obj_format ) filename = *(JString*)obj_format;
}


void JFileSuckObj::draw(JGraphics g, int x, int y, int w, int h) {
  JRect rect(x, y, w, h);
  if ( filename.length() == 0 ) {
	drawText(g, JString("TextFile"), rect);
  } else {
	drawText(g, filename, rect);
  }
}


boolean JFileSuckObj::inputAllowed(int n)
{ return !inputSet(n).last();}
JString JFileSuckObj::inputTag(int n) {
  char* tag[] = {
    "filename",
    "enabled",
  "tick" };
  return( tag[n] );
}


int JFileSuckObj::outputType( int n ) {
  if ( n == 0 ) return JStringData::id;
  return JIntegerData::id;
}
int JFileSuckObj::inputType( int n ) {
  if ( n == 0 ) return JStringData::id;
  return JIntegerData::id;
}
JString JFileSuckObj::outputTag(int n) {
  char* tag[] = {
    "text",
    "file_is_open" };
  return( tag[n] );
}


JFRect JFileSuckObj::inputArea(int n) {
  if ( n < ( 1 )) {
    return leftArea( n, 0, 1, 0.1, 0.8 );
  } else {
    return topArea( n, 1, 2, 0.25 );
  }
}


JFRect JFileSuckObj::outputArea(int n) {
  if ( n < 1 ) {
    return rightArea(n, 0, 1, 0.1, 0.8);
  } else {
    return bottomArea(n, 1, 1, 0.1, 0.8);
  }
}


int JFileSuckObj::inputFace(int n) {
  if ( n < 1 ) {
    return LEFT;
  } else {
    return TOP;
  }
}


int JFileSuckObj::outputFace(int n) {
  if ( n < 1 ) {
    return RIGHT;
  } else {
    return BOTTOM;
  }
}


JFPoint JFileSuckObj::inputPoint(int n, JLinkObj& link) {
  if (n<1) return leftPoint(n, link, 0, 1, 0.1, 0.8);
  return topPoint(n, link, 1, 2);
}


JFPoint JFileSuckObj::outputPoint(int n, JLinkObj& link) {
  if ( n < 1 ) {
    return rightPoint(n, link, 0, 1, 0.1, 0.8);
  } else {
    return bottomPoint(n, link, 1, 1, 0.1, 0.8);
  }
}


void JFileSuckObj::access(int n, JLinkObj& link, const JDataType& data) {
  if ( n == 0 ) {
    JSTRING(data) = outline;
  } else if ( n == 1 ) {
    INT(data) = file_is_open;
  }
}


bool JFileSuckObj::readLine( FILE* fp ) {
	if ( fgets( &line[0], LINE_SIZE - 1, fp ) == NULL ) {
		return( false );
	} else {
		line[LINE_SIZE - 1] = 0;
		return( true );
	}
}

#include <malloc.h>
#include <string.h>

char* newstrcpy( char* line ) {
	char* result = (char*)malloc( strlen( line ) + 1 );
	strcpy( result, line );
	return( result );
}
void JFileSuckObj::processLine() {
	outline = JString( &line[0], 2 );
	broadcast( 0 );
}

void JFileSuckObj::suckFile() {
	char* lines[1000];
	int lineIdx = 0;
	int outputLength = 0;
	while ( readLine( fp )) {
		lines[ lineIdx ] = newstrcpy( line );
		outputLength += strlen( lines[ lineIdx ] );
		lineIdx++;
	}
	char* result = (char*)malloc( outputLength + 1 );
	*result = 0;
	for ( int i = 0; i < lineIdx; i++ ) {
		strcat( result, lines[i] );
	}
	outline = JString(result);
	broadcast( 0 );
}


void JFileSuckObj::engine(int n, JLinkObj& link) {
  switch (n) {
  case FILENAME:
    link.access( JStringData( filename ));
    break;
  case ENABLED:
    link.access(JIntegerData(enabled));
	if ( enabled && filename.length() > 0 ) {
		fp = fopen( filename, "r" );
	} else {
		if ( fp != NULL ) {
			fclose( fp );
		}
		fp = NULL;
	}
	if ( fp == NULL ) {
		file_is_open = false;
	} else {
		file_is_open = true;
	}
	broadcast( 1 );
	if ( fp != NULL ) {
		suckFile();
		fclose( fp );
		fp = NULL;
	}
    break;
  case TICK:
	  link.access( JIntegerData( tick ));
	  if ( tick ) {
	  	if ( enabled && fp != NULL ) {
			if ( readLine( fp )) {
				processLine();
			} else {
				fclose( fp );
				fp = NULL;
				file_is_open = false;
				broadcast( 1 );
			}
		}
	  }
	  break;

  default:
    break;
  }
}



JArray JFileSuckObj::getProperties() {
  JArray properties;
  properties.append(JStringProperty("fileName", filename));
  return properties;
}


boolean JFileSuckObj::updateProperty(JProperty& prop) {
  if (prop.getName() == JString("fileName")) {
    filename = ((JStringProperty*)&prop)->value;
    return true;
  }
  return false;
}


