#include "XServer.h"
#include "JServerSocket.h"

#include "JLinkObj.h"
#include "JInteger.h"
#include "JIntegerData.h"
#include "JRealData.h"
#include "JBlockData.h"
#include "JBytesData.h"
#include "JCharacter.h"
#include "JStringData.h"
#include "JStringProperty.h"
#include "JIntegerProperty.h"
#include "JIntegerListProperty.h"
#include "JSocketException.h"
#include "JUnknownHostException.h"

#include <stdio.h>
#include <malloc.h>
#include <string.h>
#include <time.h>
char* newstrcpy( char* x ) {
	char* xx = (char*)malloc( strlen( x ) + 1 );
	strcpy( xx, x );
	return( xx );
}



char* theXServer = XServer().Register();		  
void XServer::writeContent(JOutputStream& os) {
  JAddObj::writeContent(os);
  putInteger(os, "port", port);
  putInteger(os, "numberOutputValues", numberOutputValues );
  putInteger(os, "htmlRefreshInterval", htmlRefreshInterval );
  putInteger(os, "debug", debug);
  putInteger(os, "synchOutputsWithInputs", synchOutputsWithInputs);
}


/**
 *  What sort of outputs does this modules produce:
 *
 *    0-(MaxValues-1):  integer
 *    MaxValues-(MaxValues*2-1):  real
 *    MaxValues*2-(MaxValues*3-1):  string
 */
int XServer::outputType( int n ) {
	int offset = n%3;
	if ( offset == 0 ) {
		return( JIntegerData::id );
	} else if ( offset == 1 ) {
		return( JRealData::id );
	} else {
		return( JStringData::id );
	}
}

void XServer::readContent(JDictionary& dict) {
  JAddObj::readContent(dict);
  port = getInteger(dict, "port");
  debug = (getInteger( dict, "debug" ) ? true : false );
  synchOutputsWithInputs = (getInteger( dict, "synchOutputsWithInputs" ) ? true : false );
  disableBrowserCaching = (getInteger( dict, "disableBrowserCaching" ) ? true : false );
  numberOutputValues = getInteger(dict, "numberOutputValues" );
  if ( numberOutputValues < 1 ) {
	  numberOutputValues = 1;
  }
  htmlRefreshInterval = getInteger(dict, "htmlRefreshInterval" );
  if (( htmlRefreshInterval < 0 ) || ( htmlRefreshInterval > 30000 )) {
	  htmlRefreshInterval = 0; // 0 means do not include "refresh" information
  }
}

const char* XServer::className() const { return "XServer";}
JObject* XServer::clone() const { 
    return new XServer(*this);
}

XServer::XServer() {
  disableBrowserCaching = false;
  port = 80;
  connected = false;
  readInput = false;
  igm = 31;
  ogm = 31;
  xprocessResult = false;
  connecting = false;
  numberOutputValues = 1;
  htmlRefreshInterval = 0;
   madeConnection = false;
   synchOutputsWithInputs = true;

   for ( int i = 0; i < MaxValues; i++ ) {
	   intOut[i] = 0;
	   floatOut[i] = 0.0;
	   stringOut[i] = JString();
   }
   for ( i = 0; i < InMaxValues; i++ ) {
	   intIn[i] = 0;
	   floatIn[i] = 0;
	   stringIn[i] = JString();
   }
}

XServer::XServer(const XServer& obj) { *this=obj;}
XServer::~XServer() { close();}



void XServer::reset() { 
  connected = false;
  data.Dereference();
  socket.close();
}

/**
 *  open
 *
 *  Purpose:
 *      Start a thread (if not already running) to connect to external program
 *      via sockets.
 */

void XServer::open( int openPort ) {
	// if a previous thread hasn't used its connection args yet, wait
	while (connecting) {
		JThread::sleep(100);
	}
  cs.lock();
    currentPort = openPort;
	connecting = true;
    thread = JThread(this);
    thread.start();
    thread.setPriority(JThread::TNORM_PRIORITY+1);
  cs.unlock();
}


/**
 *  close
 *
 *  Purpose:
 *      Stop the thread (if not already stopped) that is connected to an
 *      external program via sockets.
 */

void XServer::close() {
  cs.lock();
  if (connected) {
    connected = false;
    thread.setPriority(JThread::TNORM_PRIORITY);
    closeSocket(5);
    thread.waitFor();
    repaint();
  }
  cs.unlock();
}

void XServer::openSocket() {
    ssocket = JServerSocket(currentPort);
	if (!ssocket.listen(1)) throw new JSocketException("listen");
}
#include <winsock2.h>

boolean XServer::connectSocket() {
  if (ssocket.accept(socket)) {
	  ssocket.close();
    sis = JSocketInputStream(socket);
    sos = JSocketOutputStream(socket);
	return true;
  }
  return false;
}

void XServer::closeSocket(int count) {
  try {
	  ssocket.close();
    socket.close();
  } catch (JException *e) {
    delete e;
  }
}



/**
 *  Module characteristics associated with inputs.
 */

/**
 *  inputType
 *  
 *  Purpose:  
 *     get the input type corresponding to location
 *     All input types are int
 */

int XServer::inputType(int n) { 
	if ( n == 0 ) {
		return( JIntegerData::id );
	} else if ( n == 1 ) {
		return( JStringData::id );
	} else {
		n -= 2;
		if ( n < InMaxValues ) {
			return JIntegerData::id;
		} else if ( n < ( InMaxValues * 2 )) {
			return( JRealData::id );
		} else {
			return( JStringData::id );
		}
	}
}


/**
 *  inputTag
 *
 *  Purpose:
 *     provide a JString describing an input.
 */

JString XServer::inputTag(int n) {
  char* tag[] = {

      // top inputs
    "readInput",
	"templateText",
		
		// side inputs
    "int_0",
	"int_1",
	"int_2",
	"int_3",
	"int_4",
	"int_5",
	"int_6",
	"int_7",
	"int_8",
	"int_9",
	"real_0",
	"real_1",
	"real_2",
	"real_3",
	"real_4",
	"real_5",
	"real_6",
	"real_7",
	"real_8",
	"real_9",
	"string_0",
	"string_1",
	"string_2",
	"string_3",
	"string_4",
	"string_5",
	"string_6",
	"string_7",
	"string_8",
	"string_9"
  };
  return( tag[n] );
 }



boolean XServer::inputAllowed(int n) { 
        return !inputSet(n).last();
}


JFRect XServer::inputArea(int n) {
	if (( n == 0 ) || ( n == 1 )) {
		return topArea( n, 0, 2, 0.25 );
	} else {
		return( leftArea( n, 2, 31, 0.1, 0.8 ));
	}
}


JFRect XServer::outputArea(int n) {
//	if ( n < 30 )
  return rightArea(n, 0, numberOutputValues*3, 0.1, 0.8);
//	else return( bottomArea( n, 30, 1 ));
}


int XServer::inputFace(int n) {
	if ( n < 2 ) return TOP;
	else return LEFT;
}


int XServer::outputFace(int n ) {
	if ( n < MaxValues*3 ) {
		return( RIGHT );
	} else {
		return( BOTTOM );
	}
}

JFPoint XServer::inputPoint(int n, JLinkObj& link) {
	if (n < 2)
  return topPoint(n, link, 0, 2 );
	else return(leftPoint(n, link, 2, 31, 0.1, 0.8 ));
}


JFPoint XServer::outputPoint(int n, JLinkObj& link) {
	if ( n < MaxValues*3 )
  return rightPoint(n, link, 0, MaxValues*3, 0.1, 0.8);
	else return bottomPoint(n, link, MaxValues*3, 1);
}

/**
 *  Module characteristics associated with outputs
 */



void XServer::paint(JGraphics g, double dx, double dy, JRegion& rgn, double scale) {
  JAddObj::paint(g, dx, dy, rgn, scale);
  JRect rect = getIExtent(dx, dy, scale);
  int kw = depth2*3;
  if ((rect.width > kw) && (rect.height > kw)) {
    if (madeConnection) g.setJColor(JColor::green);
    else g.setJColor(JColor::red);
    g.fillRect(rect.x+depth2, rect.y+depth2, depth2, depth2);
  }
}

void XServer::draw(JGraphics g, int x, int y, int w, int h) {
  int dh = h/3;
  int dy = h/12;
  JRect rect1(x, y+dy, w, dh);
  JRect rect2(x, y+h/2, w, dh);
  drawText(g, "XML Server", rect1);
  char buf[50];
  sprintf( buf, "port: %d", port );
  drawText(g, buf, rect2);
}




/**
 *  engine
 *
 *  Purpose:
 *     The main processing engine for this module, called whenever
 *     data is received on an incoming visual link.
 */
void XServer::engine(int n, JLinkObj& link) {
    switch (n) {
    case READINPUT:
        link.access(JIntegerData(readInput));
        if (readInput && !connected) {
			xprocessResult = true;
			open(port);
		} else if (!readInput && connected) { // && wroteData) {
			close();
		}
        break;
	case TEMPLATETEXT:
		link.access(JStringData(templateText));
		if ( debug ) {
			JComponent::debug( "templateText is:" );
			JComponent::debug( templateText );
		}
		createSprintfFormat();
		if ( debug ) {
			JComponent::debug( newstrcpy( sprintf_format ));
		}
		// temporary for testing
		createServerOutput();
		break;
    default:
        break;
    }
	if ( n > TEMPLATETEXT ) {
		n -= 2;
		if ( n < InMaxValues ) {
			link.access( JIntegerData( intIn[ n ] ));
			if ( synchOutputsWithInputs ) {
				intOut[ n ] = intIn[ n ];
				broadcast( n*3 );
			}
		} else if ( n < ( InMaxValues * 2 )) {
			link.access( JRealData( floatIn[ n - InMaxValues ] ));
			if ( synchOutputsWithInputs ) {
				floatOut[ n - InMaxValues ] = floatIn[ n - InMaxValues ];
				broadcast( ( n - InMaxValues ) * 3 + 1 );
			}
		} else if ( n < ( InMaxValues * 3 )) {
			link.access( JStringData( stringIn[ n - InMaxValues * 2 ] ));
			if ( synchOutputsWithInputs ) {
				stringOut[ n - MaxValues * 2 ] = stringIn[ n - MaxValues * 2 ];
				broadcast( ( n - InMaxValues ) * 3 + 2 );
			}
		}
	}
}


JArray XServer::getProperties() {
  JArray properties = JAddObj::getProperties();
  properties.append(JIntegerProperty("port", port, 80, 10000));
  properties.append(JIntegerListProperty("disableBrowserCaching", disableBrowserCaching, JIntegerListProperty::booleanTag ));
  properties.append(JIntegerProperty("numberOutputValues", numberOutputValues, 1, MaxValues));
  properties.append(JIntegerProperty("htmlRefreshInterval", htmlRefreshInterval, 0, 30000));
  properties.append(JIntegerListProperty("debug", debug, JIntegerListProperty::booleanTag ));
  properties.append(JIntegerListProperty("synchOutputsWithInputs", synchOutputsWithInputs, JIntegerListProperty::booleanTag ));
  return properties;
}

boolean XServer::updateProperty(JProperty& prop) {
  if (JAddObj::updateProperty(prop)) return true;
  boolean changed = false;
  if (prop.getName() == JString("port")) {
    port = ((JIntegerProperty*)&prop)->value;
    changed = true;
  } else if ( prop.getName() == JString("disableBrowserCaching" )) {
	  disableBrowserCaching = ((JIntegerProperty*)&prop)->value ? true : false;
	  changed = true;
  } else if (prop.getName() == JString("numberOutputValues")) {
    numberOutputValues = ((JIntegerProperty*)&prop)->value;
    changed = true;
  } else if (prop.getName() == JString("htmlRefreshInterval")) {
    htmlRefreshInterval = ((JIntegerProperty*)&prop)->value;
    changed = true;
  } else if ( prop.getName() == JString("debug" )) {
	  debug = ((JIntegerProperty*)&prop)->value ? true : false;
	  changed = true;
  } else if ( prop.getName() == JString("synchOutputsWithInputs" )) {
	  synchOutputsWithInputs = ((JIntegerProperty*)&prop)->value ? true : false;
	  changed = true;
  }
  if (changed) repaint();
  return changed;
}


void XServer::initData() {
	outIdx = 0;
	workingOnValue = false;
	wroteData = false;
}






int XServer::processResult( char* b, int size, int start ) {
	int result = 0;
	for ( int i = start; i < size; i++ ) {
		if (( b[ i ] >= '0' ) && ( b[ i ] <= '9' )) {
			result = result * 10 + ( b[i] - '0' );
		} else {
			return result;
		}
	}
	return -1;
}



JString XServer::outputTag(int n) {
  char* tag[] = {
    "int_0",
    "real_0",
	"string_0",
    "int_1",
    "real_1",
	"string_1",
    "int_2",
    "real_2",
	"string_2",
    "int_3",
	"real_3",
	"string_3",
    "int_4",
	"real_4",
	"string_4",
    "int_5",
	"real_5",
	"string_5",
    "int_6",
	"real_6",
	"string_6",
    "int_7",
	"real_7",
	"string_7",
    "int_8",
	"real_8",
	"string_8",
    "int_9",
	"real_9",
	"string_9",
    "int_10",
    "real_10",
	"string_10",
    "int_11",
    "real_11",
	"string_11",
    "int_12",
    "real_12",
	"string_12",
    "int_13",
	"real_13",
	"string_13",
    "int_14",
	"real_14",
	"string_14",
    "int_15",
	"real_15",
	"string_15",
    "int_16",
	"real_16",
	"string_16",
    "int_17",
	"real_17",
	"string_17",
    "int_18",
	"real_18",
	"string_18",
    "int_19",
	"real_19",
	"string_19",
    "int_20",
    "real_20",
	"string_20",
    "int_21",
    "real_21",
	"string_21",
    "int_22",
    "real_22",
	"string_22",
    "int_23",
	"real_23",
	"string_23",
    "int_24",
	"real_24",
	"string_24",
    "int_25",
	"real_25",
	"string_25",
    "int_26",
	"real_26",
	"string_26",
    "int_27",
	"real_27",
	"string_27",
    "int_28",
	"real_28",
	"string_28",
    "int_29",
	"real_29",
	"string_29",
    "int_30",
    "real_30",
	"string_30",
    "int_31",
    "real_31",
	"string_31",
    "int_32",
    "real_32",
	"string_32",
    "int_33",
	"real_33",
	"string_33",
    "int_34",
	"real_34",
	"string_34",
    "int_35",
	"real_35",
	"string_35",
    "int_36",
	"real_36",
	"string_36",
    "int_37",
	"real_37",
	"string_37",
    "int_38",
	"real_38",
	"string_38",
    "int_39",
	"real_39",
	"string_39",
    "int_40",
    "real_40",
	"string_40",
    "int_41",
    "real_41",
	"string_41",
    "int_42",
    "real_42",
	"string_42",
    "int_43",
	"real_43",
	"string_43",
    "int_44",
	"real_44",
	"string_44",
    "int_45",
	"real_45",
	"string_45",
    "int_46",
	"real_46",
	"string_46",
    "int_47",
	"real_47",
	"string_47",
    "int_48",
	"real_48",
	"string_48",
    "int_49",
	"real_49",
	"string_49",
    "int_50",
    "real_50",
	"string_50",
    "int_51",
    "real_51",
	"string_51",
    "int_52",
    "real_52",
	"string_52",
    "int_53",
	"real_53",
	"string_53",
    "int_54",
	"real_54",
	"string_54",
    "int_55",
	"real_55",
	"string_55",
    "int_56",
	"real_56",
	"string_56",
    "int_57",
	"real_57",
	"string_57",
    "int_58",
	"real_58",
	"string_58",
    "int_59",
	"real_59",
	"string_59",
    "int_60",
    "real_60",
	"string_60",
    "int_61",
    "real_61",
	"string_61",
    "int_62",
    "real_62",
	"string_62",
    "int_63",
	"real_63",
	"string_63",
    "int_64",
	"real_64",
	"string_64",
    "int_65",
	"real_65",
	"string_65",
    "int_66",
	"real_66",
	"string_66",
    "int_67",
	"real_67",
	"string_67",
    "int_68",
	"real_68",
	"string_68",
    "int_69",
	"real_69",
	"string_69",
    "int_70",
    "real_70",
	"string_70",
    "int_71",
    "real_71",
	"string_71",
    "int_72",
    "real_72",
	"string_72",
    "int_73",
	"real_73",
	"string_73",
    "int_74",
	"real_74",
	"string_74",
    "int_75",
	"real_75",
	"string_75",
    "int_76",
	"real_76",
	"string_76",
    "int_77",
	"real_77",
	"string_77",
    "int_78",
	"real_78",
	"string_78",
    "int_79",
	"real_79",
	"string_79",
    "int_80",
    "real_80",
	"string_80",
    "int_81",
    "real_81",
	"string_81",
    "int_82",
    "real_82",
	"string_82",
    "int_83",
	"real_83",
	"string_83",
    "int_84",
	"real_84",
	"string_84",
    "int_85",
	"real_85",
	"string_85",
    "int_86",
	"real_86",
	"string_86",
    "int_87",
	"real_87",
	"string_87",
    "int_88",
	"real_88",
	"string_88",
    "int_89",
	"real_89",
	"string_89",
    "int_90",
    "real_90",
	"string_90",
    "int_91",
    "real_91",
	"string_91",
    "int_92",
    "real_92",
	"string_92",
    "int_93",
	"real_93",
	"string_93",
    "int_94",
	"real_94",
	"string_94",
    "int_95",
	"real_95",
	"string_95",
    "int_96",
	"real_96",
	"string_96",
    "int_97",
	"real_97",
	"string_97",
    "int_98",
	"real_98",
	"string_98",
    "int_99",
	"real_99",
	"string_99",
  };
  return( tag[n] );
}



void XServer::writeData() {
	wroteData = true;
	for ( int i = 0; i < outIdx; i++ ) {
		broadcast( i );
	}
}

void XServer::access(int n, JLinkObj& link, const JDataType& data) {
	int typeIndicator = n%3;
	int typeOffset = n/3;
	if ( typeIndicator == 0 ) {
     INT(data) = intOut[typeOffset];
	} else if ( typeIndicator == 1 ) {
		DOUBLE(data) = (double)floatOut[typeOffset];
	} else {
		JSTRING(data) = stringOut[typeOffset];
	}
}

bool doubleNewLine( char* s ) {
	int nlcount = 0;
	while ( *s ) {
		if (( *s == '\n' ) || ( *s == '\r' )) {
			nlcount++;
			if ( nlcount == 2 ) {
				return( true );
			}
		} else {
			nlcount = 0;
		}
		s++;
	}
	return( false );
}


/**
 *  processInputStream
 *
 *  Purpose:
 */
static int counter = 0;
static char dataBuf[10000];

char* getReq( char* s ) {
	char result[200];
	char resultIdx = 0;
	while ( *s && ( *s != ' ' )) {
		s++;
	}
	if ( *s == ' ' ) {
		s++;
	}
	while ( *s && ( *s != ' ' )) {
		result[ resultIdx++ ] = *s++;
		result[ resultIdx ] = 0;
	}
	return( newstrcpy( result ));
}

void XServer::setIntOutput( char* scanner ) {
	if (( *scanner >= '0' ) && ( *scanner <= '9' )) {
		int whichOutput = *scanner - '0';
		scanner++;
		if ( *scanner == '=' ) {
			scanner++;
			intOut[ whichOutput ] = atoi( scanner );
			broadcast( whichOutput );
		}
	}
}

void XServer::setRealOutput( char* scanner ) {
	if (( *scanner >= '0' ) && ( *scanner <= '9' )) {
		int whichOutput = *scanner - '0';
		scanner++;
		if ( *scanner == '=' ) {
			scanner++;
			floatOut[ whichOutput ] = atof( scanner );
			broadcast( whichOutput + MaxValues );
		}
	}
}
void XServer::setStringOutput( char* scanner ) {
		if (( *scanner >= '0' ) && ( *scanner <= '9' )) {
		int whichOutput = *scanner - '0';
		scanner++;
		if ( *scanner == '=' ) {
			scanner++;
			char sbuf[100];
			int sbufIdx = 0;
			while ( *scanner && (*scanner != '&' )) {
				sbuf[sbufIdx++] = *scanner++;
				sbuf[sbufIdx] = 0;
				if ( sbufIdx == 99 ) {
					break;
				}
			}
			stringOut[ whichOutput ] = JString( newstrcpy( sbuf ));

			broadcast( whichOutput + MaxValues * 2 );
		}
	}

}

void XServer::setOutputs() {
	char* scanner = requestedURL;
	scanner += 5;
	while ( *scanner ) {
		if ( *scanner == 'i' ) {
			setIntOutput( scanner + 1 );
		} else if ( *scanner == 'r' ) {
			setRealOutput( scanner + 1 );
		} else if ( *scanner == 's' ) {
			setStringOutput( scanner + 1 );
		}
		while ( *scanner && ( *scanner != '&' )) {
			scanner++;
		}
		if ( *scanner == '&' ) {
			scanner++;
		}
	}
}

static char* DISABLE_CACHING_HEADER = "<META HTTP-EQUIV=\"Cache-Control\" CONTENT=\"no-cache\"> \n<META HTTP-EQUIV=\"Pragma\" CONTENT=\"no-cache\"> \n<meta http-equiv=\"expires\" content=\"FRI, 13 APR 1999 01:00:00 GMT\"> ";
int debugcounter = 0;
int XServer::processInputStream() {
  initData();
  counter = 0;
  dataBuf[0] = 0;
  while (connected > true) {
    try {
	  int socketHandle = sos;
	  counter++;
	  if ( counter == 400 ) {
		  exit( 0 );
	  }
	  char buf[10000];
	  if ( debug ) {
		  sprintf( buf, "socketHandle is %d", socketHandle );
		  JComponent::debug(newstrcpy(buf ));
	  }
	  int result = recv( socketHandle, &buf[0], 10000, 0 );
	  if ( debug ) {
		  JComponent::debug("got result");
	  }
	  if ( result > 0 ) {
		buf[result] = 0;
		requestedURL = JString(getReq(buf));
		if ( debug ) {
			JComponent::debug( "requestedURL is:" );
			JComponent::debug( requestedURL );
		}
		broadcast(MaxValues*3);
		madeConnection = true;
		if (( strlen( dataBuf ) + result ) < 5000 ) {
			strcat( dataBuf, buf );
		}
		if ( debug ) {
			char xbuf[10000];
			sprintf(xbuf, "dataBuf now: %s", dataBuf );
			JComponent::debug(newstrcpy(xbuf));
		}
		
	  } else if ( result == -1 ) {
		  if ( debug ) {
			JComponent::debug("-1");
		  }
		  connected = false;
	  } else if ( result == 0 ) {
		  if ( debug ) {
			JComponent::debug( "z result" );
		  }
		  connected=false;
	  } else {
		  if ( debug ) {
			JComponent::debug("hmm. weird result" );
		  }
	  }
	  if ( doubleNewLine( dataBuf )) {
		  break;
	  }
    } catch (JException *e) {
        JComponent::debug( "input stream thread got exception" );
		//writeData();
	 
      delete e;
      return -1;
    }
  }
  	int outFd = sos;
	char outBuf[500];
	strcpy( outBuf, "nothing requested: i ==> requests ints, r ==> requests reals, s ==> requests strings<br>" );
	strcat( outBuf, "Requested URL was '" );
	strcat( outBuf, requestedURL );
	strcat( outBuf, "'<br>" );
	if ( strncmp( (char*)requestedURL, "/h", 2 ) == 0 ) {
		createServerOutput();
		strcpy( outBuf, "HTTP/1.1 200 OK\r\n" );
		char lbuf[10];
		sprintf( lbuf, "%d\r\n", strlen( serverOutput ) + ( disableBrowserCaching ? strlen( DISABLE_CACHING_HEADER ) : 0 ));
		strcat( outBuf, "Content-Length: " );
		strcat( outBuf, lbuf );
		strcat( outBuf, "Content-Type: text/html\r\n\r\n" );
		if ( disableBrowserCaching ) {
			strcat( outBuf, DISABLE_CACHING_HEADER );
		}
		strcat( outBuf, serverOutput );
	} else if ( requestedURL == "/x" ) {
/*		createServerOutput();
		strcpy( outBuf, "HTTP/1.1 200 OK\r\n" );
		strcat( outBuf, "Connection: Keep-Alive\r\n" );
		strcat( outBuf, "Transfer-Encoding: chunked\r\n" );

		strcat( outBuf, "Content-Type: text/xml\r\n\r\n" );
		char lbuf[10];
		sprintf( lbuf, "%x \r\n", strlen( serverOutput ));
		strcat( outBuf, lbuf );
*/
		strcpy( outBuf, "HTTP/1.1 200 OK\r\n" );
		//strcat( outBuf, "Date: Sun, 18 Jan 2004 19:15:41 GMT\r\n" );
		//strcat( outBuf, "Server: Apache/1.3.29\r\n" );
		//strcat( outBuf, "Keep-Alive: timeout=5, max=100\r\n" );
		//strcat( outBuf, "Connection: Keep-Alive\r\n" );
		strcat( outBuf, "Transfer-Encoding: chunked\r\n" );
		strcat( outBuf, "Content-Type: text/xml\r\n\r\n" );
		strcat( outBuf, "7c \r\n" );
		strcat( outBuf, "<?xml version=\"1.0\"?>\n" );
		strcat( outBuf, "<methodResponse>\n" );
		strcat( outBuf, "<params>\n<param><value><string></string></value></param>\n" );
		strcat( outBuf, "</params>\n" );
		strcat( outBuf, "</methodResponse>\n\r\n" );
		//rcat( outBuf, serverOutput );
	} else if ( requestedURL == "/i" ) {
		sprintf( outBuf, "%d,%d,%d,%d,%d,%d,%d,%d,%d,%d", intIn[0], intIn[1], 
			intIn[2], intIn[3], intIn[4], intIn[5], intIn[6], intIn[7],
			intIn[8], intIn[9] );
	} else if ( requestedURL == "/r" ) {
		sprintf( outBuf, "%f,%f,%f,%f,%f,%f,%f,%f,%f,%f", floatIn[0], floatIn[1],
			floatIn[2], floatIn[3], floatIn[4], floatIn[5], floatIn[6],
			floatIn[7], floatIn[8], floatIn[9] );
	} else if ( requestedURL == "/s" ) {
		sprintf( outBuf, "%s,%s,%s,%s,%s,%s,%s,%s,%s,%s", (char*)stringIn[0],
			(char*)stringIn[1], (char*)stringIn[2], (char*)stringIn[3],
			(char*)stringIn[4], (char*)stringIn[5], (char*)stringIn[6],
			(char*)stringIn[7], (char*)stringIn[8], (char*)stringIn[9] );
	} else if ( requestedURL == "/a" ) {
		sprintf( outBuf, "%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%f,%f,%f,%f,%f,%f,%f,%f,%f,%f,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s", 
			intIn[0], intIn[1], 
			intIn[2], intIn[3], intIn[4], intIn[5], intIn[6], intIn[7],
			intIn[8], intIn[9], floatIn[0], floatIn[1],
			floatIn[2], floatIn[3], floatIn[4], floatIn[5], floatIn[6],
			floatIn[7], floatIn[8], floatIn[9], (char*)stringIn[0],
			(char*)stringIn[1], (char*)stringIn[2], (char*)stringIn[3],
			(char*)stringIn[4], (char*)stringIn[5], (char*)stringIn[6],
			(char*)stringIn[7], (char*)stringIn[8], (char*)stringIn[9] );
	} else if ( strncmp( requestedURL, "/set?", 5 ) == 0 ) {
		setOutputs();
		strcpy( outBuf, "values set" );
	}
	if ( debug ) {
		JComponent::debug( newstrcpy( outBuf ));
	}
	int result = send( outFd, outBuf, strlen( outBuf ), 0 );
	if ( result == -1 ) {
		closeSocket( 7 );
		connected = false;
	}
	if ( debug ) {
		char dbuf[100];
		sprintf( dbuf, "send result is %d", result );
		JComponent::debug( newstrcpy( dbuf ));
	}
	if ( result == -1 ) {
		return( -1 );
	}
	return 0;

}


void XServer::run() {
	initData();
reconnect:
	try {
		openSocket();
	} catch (JException *e) {
		JComponent::debug( "got exception" );
		JComponent::debug( e->getMessage() );
		delete e;
		closeSocket(1);
		connecting = false;
		return;
	}
	connected = true;
	repaint();
	if ( debug ) {
		JComponent::debug("try to connect");
	}
	try {
		while (connected) {
			if (connectSocket()) break;
			JThread::sleep(100);
		}
		if ( debug ) {
			JComponent::debug( "connected!" );
		}
		connecting = false;
		if (!connected) {
			closeSocket(2);
			connected = false;
			repaint();
			return;
		}
	} catch (JException *e) {
		delete e;
		closeSocket(3);
		repaint();
		return;
	}
	connected++;
	if ( debug ) {
		JComponent::debug("connected");
	}
	madeConnection = true;
	repaint();
	while ( readInput ) {			
		if ( xprocessResult ) {
			if ( processInputStream() == -1 ) {
				if ( readInput ) {
					goto reconnect;
				}
			}
		}
		if (connected > true) {
			//JThread::sleep(3000);
			closeSocket(4);
			goto reconnect;
		}
		repaint();
		connected = false;
	}
}


void XServer::mapValue( int mapperType, int mapperOffset, char* sprintf_value ) {
	switch ( mapperType ) {
	case INT_MAPPER:
		sprintf( sprintf_value, "%d", intOut[ mapperOffset ] );
		break;
	case REAL_MAPPER:
		sprintf( sprintf_value, "%f", floatOut[ mapperOffset ] );
		break;
	case STRING_MAPPER:
		sprintf( sprintf_value, "%s", stringOut[ mapperOffset ] );
		break;
	}
}

void XServer::createMapper( int* mapperType, int* mapperOffset, char* templateIndicator ) {
	if (( *templateIndicator == 'i' ) || ( *templateIndicator == 'I' )) {
		*mapperType = INT_MAPPER;
		*mapperOffset = atoi( templateIndicator + 1 );
	} else if (( *templateIndicator == 'r' ) || ( *templateIndicator == 'R' )) {
		*mapperType = REAL_MAPPER;
		*mapperOffset = atoi( templateIndicator + 1 );
	} else if (( *templateIndicator == 's' ) || ( *templateIndicator == 'S' )) {
		*mapperType = STRING_MAPPER;
		*mapperOffset = atoi( templateIndicator + 1 );
	} else {
		JComponent::debug( "Unknown variable referenced in template:" );
		JComponent::debug( newstrcpy( templateIndicator ));
	}
}


//
// createSprintfFormat
//
//    Initializes the "sprintf_format" variable and all the associated mapping
//    between template variables and module variables (mapping information kept
//    in two arrays: "mapperTypes" and "mapperOffsets").
//
//    Mapping information and this format are used by the "createServerOutput"
//    method below.
//

void XServer::createSprintfFormat() {
	// initialize outputs
	sprintf_format[0] = 0;
	for ( int i = 0; i < MaxTemplateVars; i++ ) {
		mapperTypes[i] = NO_MAPPER;
		mapperOffsets[i] = 0;
	}

	// scan "templateText" to build the "sprintf_format" and assign mapper info 
	int variableCounter = 0;
	char* scanner = (char*)templateText;
	char* outputScanner = &sprintf_format[0];
	bool isVariable = true;
	while ( *scanner ) {
		if ( *scanner == '\\' ) {
			isVariable = false;
		} else if (( *scanner == '$' ) && isVariable ) {
			scanner++;
			createMapper( &mapperTypes[ variableCounter ], &mapperOffsets[ variableCounter ], scanner );
			scanner++;
			if (( *scanner >= '0' ) && ( *scanner <= '9' )) {
				scanner++;
			}
			strcat( sprintf_format, "%s" );
			outputScanner = &sprintf_format[ strlen( sprintf_format ) ];
			variableCounter++;
		} else {
			isVariable = true;
			*outputScanner++ = *scanner++;
			*outputScanner = 0;
		}
	}
}


//
//  createServerOutput
//
//    Initializes the "serverOutput" variable to have actual server output text.
//
//    Assumes the "sprintf_format" variable and mapper variables to have been initialized 
//    by "createSprintFormat()" defined above.
//
//    This method uses the mapper values to initialize the "sprintf_values" strings,
//    which are passed as values to the sprintf that uses "sprintf_format"
//
void XServer::createServerOutput() {
	// initialize the output strings (since not all will be given values,
	// but all must be passed to sprintf
	for ( int i = 0; i < MaxTemplateVars; i++ ) {
		sprintf_values[i][0] = 0;
	}

	// set up all the values using the mappers
	for ( i = 0; i < MaxTemplateVars; i++ ) {
		mapValue( mapperTypes[i], mapperOffsets[i], &sprintf_values[i][0] );
	}

	// create server output using these newly created values
	sprintf( serverOutput, sprintf_format, sprintf_values[0], sprintf_values[1],
		sprintf_values[2], sprintf_values[3], sprintf_values[4], sprintf_values[5],
		sprintf_values[6], sprintf_values[7], sprintf_values[8], sprintf_values[9],
		sprintf_values[10], sprintf_values[11], sprintf_values[12], sprintf_values[13],
		sprintf_values[14], sprintf_values[15], sprintf_values[16], sprintf_values[17],
		sprintf_values[18], sprintf_values[19], sprintf_values[20], sprintf_values[21],
		sprintf_values[22], sprintf_values[23], sprintf_values[24], sprintf_values[25],
		sprintf_values[26], sprintf_values[27], sprintf_values[28], sprintf_values[29],
		sprintf_values[30], sprintf_values[31], sprintf_values[32], sprintf_values[33],
		sprintf_values[34], sprintf_values[35], sprintf_values[36], sprintf_values[37],
		sprintf_values[38], sprintf_values[39], sprintf_values[40], sprintf_values[41],
		sprintf_values[42], sprintf_values[43], sprintf_values[44], sprintf_values[45],
		sprintf_values[46], sprintf_values[47], sprintf_values[48], sprintf_values[49],
		sprintf_values[50], sprintf_values[51], sprintf_values[52], sprintf_values[53],
		sprintf_values[54], sprintf_values[55], sprintf_values[56], sprintf_values[57],
		sprintf_values[58], sprintf_values[59], sprintf_values[60], sprintf_values[61],
		sprintf_values[62], sprintf_values[63], sprintf_values[64], sprintf_values[65],
		sprintf_values[66], sprintf_values[67], sprintf_values[68], sprintf_values[69],
		sprintf_values[70], sprintf_values[71], sprintf_values[72], sprintf_values[73],
		sprintf_values[74], sprintf_values[75], sprintf_values[76], sprintf_values[77],
		sprintf_values[78], sprintf_values[79], sprintf_values[80], sprintf_values[81],
		sprintf_values[82], sprintf_values[83], sprintf_values[84], sprintf_values[85],
		sprintf_values[86], sprintf_values[87], sprintf_values[88], sprintf_values[89],
		sprintf_values[90], sprintf_values[91], sprintf_values[92], sprintf_values[93],
		sprintf_values[94], sprintf_values[95], sprintf_values[96], sprintf_values[97],
		sprintf_values[98], sprintf_values[99] );

	if ( debug ) {
		JComponent::debug( "ServerOutput:" );
		JComponent::debug( serverOutput );
	}
}