#include "gps.h"
#include <math.h>
#include <stdlib.h>
#include <string.h>

int digitCount( char* s, int maxReturn ) {
	int count = 0;
	while (( *s >= '0' ) && ( *s <= '9' )) {
		count++;
		s++;
		if ( count == maxReturn ) {
			break;
		}
	}
	return( count );
}

char* skipPast( char* s, char destination ) {
	while ( *s && ( *s != destination )) {
		s++;
	}
	if ( *s == destination ) {
		return( s + 1 );
	}
	return( s );
}

int atoiWithDigitCount( char* s, int limit ) {
	char buf[20];
	strncpy( buf, s, limit );
	return( atoi( buf ));
}

// for converting a string longitude and latitude into these ints, 
// something like signing North/South as +/- (same for East/West)
// then multiplying number by 10000, so that real "NN.MMMMM" would
// become an int "NNMMMMM"

int GPS_2_int( char* latitude_or_longitude ) {
	int leftPart = atoi( latitude_or_longitude );
	latitude_or_longitude = skipPast( latitude_or_longitude, '.' );

	int rightPart = atoiWithDigitCount( latitude_or_longitude, 5 );
	int timesTenMultiplier = 5 - digitCount( latitude_or_longitude, 5 );
	for ( int i = 0; i < timesTenMultiplier; i++ ) {
		rightPart *= 10;
	}
	latitude_or_longitude = skipPast( latitude_or_longitude, ' ' );

	// conventions:  E/W maps to +/-, N/S maps to +/-
	int sign = 1;
	if (( *latitude_or_longitude == 'w' ) || ( *latitude_or_longitude == 'W' )) {
		sign = -1;
	} else if (( *latitude_or_longitude == 'S' ) || ( *latitude_or_longitude == 'S' )) {
		sign = -1;
	}
	return( sign * ( leftPart * 100000 + rightPart ));
}


GPS_map::GPS_map() {
	init( 0, 0, 0, 0, "", 1, 1 );
}

void GPS_map::init( int x_longitude_start, int x_longitude_end, 
				 int y_latitude_start, int y_latitude_end, 
				 char* imageFile, 
				 int pixel_width, int pixel_height ) {
	this->x_longitude_start = x_longitude_start;
	this->x_longitude_end = x_longitude_end;
	this->y_latitude_start = y_latitude_start;
	this->y_latitude_end = y_latitude_end;
	this->pixel_width = pixel_width;
	this->pixel_height = pixel_height;

	x_width = abs( x_longitude_end - x_longitude_start );
	x_positive_increasing = ( x_longitude_end > x_longitude_start ) ? true : false;

	y_height = abs( y_latitude_end - y_latitude_start );
	y_positive_increasing = ( y_latitude_end > y_latitude_start ) ? true : false;

	x_per_pixel = x_width / pixel_width;
	x_remainder = x_width % pixel_width;

	y_per_pixel = y_height / pixel_height;
	y_remainder = y_height % pixel_height;

	x_boundary = ( x_per_pixel + 1 ) * x_remainder;
	y_boundary = ( y_per_pixel + 1 ) * y_remainder;
}

int GPS_map::longitude_to_xPixel( int x_longitude ) {
	int x_offset = abs( x_longitude - x_longitude_start );
	if ( x_offset < x_boundary ) {
		return( x_offset / ( x_per_pixel + 1 ));
	} else {
		return( x_remainder + ( x_offset - x_boundary )/x_per_pixel );
	}
}

int GPS_map::xPixel_to_longitude( int xPixel ) {
	if ( xPixel < x_remainder ) {
		return( xPixel * ( x_per_pixel + 1 ));
	} else {
		return( x_boundary + ( xPixel - x_remainder ) * x_per_pixel );
	}
}

int GPS_map::latitude_to_yPixel( int y_latitude ) {
	int y_offset = abs( y_latitude - y_latitude_start );
	if ( y_offset < y_boundary ) {
		return( y_offset / ( y_per_pixel + 1 ));
	} else {
		return( y_remainder + ( y_offset - y_boundary ) / y_per_pixel );
	}
}

int GPS_map::yPixel_to_latitude( int yPixel ) {
	if ( yPixel < y_remainder ) {
		return( yPixel * ( y_per_pixel + 1 ));
	} else {
		return( y_boundary + ( yPixel - y_remainder ) * y_per_pixel );
	}
}

int x_failure_code = 0;
bool GPS_map::x_isActive( int x_longitude ) {
	if ( x_positive_increasing ) {
		if ( x_longitude < x_longitude_start ) {
			x_failure_code = 1;
			return( false );
		}
		if ( x_longitude > x_longitude_end ) {
			x_failure_code = 2;
			return( false );
		}
	} else {
		if ( x_longitude > x_longitude_start ) {
			x_failure_code = 3;
			return( false );
		}
		if ( x_longitude < x_longitude_end ) {
			x_failure_code = 4;
			return( false );
		}
	}
	x_failure_code = 0;
	return( true );
}

int y_failure_code = 0;
bool GPS_map::y_isActive( int y_latitude ) {
	if ( y_positive_increasing ) {
		if ( y_latitude < y_latitude_start ) {
			y_failure_code = 1;
			return( false );
		}
		if ( y_latitude > y_latitude_end ) {
			y_failure_code = 2;
			return( false );
		}
	} else {
		if ( y_latitude > y_latitude_start ) {
			y_failure_code = 3;
			return( false );
		}
		if ( y_latitude < y_latitude_end ) {
			y_failure_code = 4;
			return( false );
		}
	}
	y_failure_code = 0;
	return( true );
}

bool GPS_map::isActive( int x_longitude, int y_latitude ) {
	return( x_isActive( x_longitude ) && y_isActive( y_latitude ));
}

#include <stdio.h>

char* GPS_map::toString() {
	static char buf[100];
	sprintf( buf, "GPS_map: long %d to %d, lat %d to %d, failure_codes %d, %d", x_longitude_start, x_longitude_end, y_latitude_start, y_latitude_end, x_failure_code, y_failure_code );
	return( &buf[0] );
}

