//
// GPS data is expected to come in as strings, e.g. "118.12345 E"
//
// To convert this to pixel coordinates involves two steps:
//
// 1. converting incoming string to an integer representation.
//    This is done by the GPS_2_int() function.
//
// 2. performing the actual map-specific conversion.
//    This is done by GPS_map below.
//

int GPS_2_int( char* latitude_or_longitude );

class __declspec( dllexport ) GPS_map {
public:
	GPS_map();
	void init(int x_longitude_start, int x_longitude_end,
		int y_latitude_start, int y_latitude_end,
		char* imageFile,
		int pixel_width, int pixel_height );

	int longitude_to_xPixel( int x_longitude );
	int xPixel_to_longitude( int xPixel );

	int latitude_to_yPixel( int y_latitude );
	int yPixel_to_latitude( int yPixel );

	bool isActive( int x_longitude, int y_latitude );
	char* toString();
private:
	bool x_isActive( int x_longitude );
	bool y_isActive( int y_latitude );

	int x_longitude_start;
	int x_longitude_end;
	int x_width;
	bool x_positive_increasing;

	int y_latitude_start;
	int y_latitude_end;
	int y_height;
	bool y_positive_increasing;

	int pixel_width;  // width of image map in pixels
	int x_per_pixel;  // number of x units per pixel
	int x_remainder;  // number of x units remaining when longitude range divided by pixel_width

	int pixel_height; // height of image map in pixels
	int y_per_pixel;  // number of y units per pixel
	int y_remainder;  // number of y units remaining when latitude range divided by pixel_height

	int x_boundary;   // the point at which "x_per_pixel" becomes valid when mapping to pixel coords
	int y_boundary;   // the point at which "y_per_pixel" becomes valid when mapping to pixel coords
};
