#include "gps.h"
#include "neattools_mapper.h"
#include <stdio.h>
#include <string.h>

neattools_mapper::neattools_mapper() {
	lastLongitude[0] = 0;
	lastLatitude[0] = 0;
	lastIntLongitude = 0;
	lastIntLatitude = 0;
	init( "", "", "", "", "", 1, 1 );
}

void neattools_mapper::init( char* xs_longitude_start, char* xs_longitude_end,
							char* ys_latitude_start, char* ys_latitude_end,
							char* imageFile, 
							int pixel_width, int pixel_height ) {
	int x_longitude_start = GPS_2_int( xs_longitude_start );
	int x_longitude_end = GPS_2_int( xs_longitude_end );
	int y_latitude_start = GPS_2_int( ys_latitude_start );
	int y_latitude_end = GPS_2_int( ys_latitude_end );
	gpsMap.init( x_longitude_start, x_longitude_end, y_latitude_start, y_latitude_end,
		imageFile, pixel_width, pixel_height );
	pixelX = 0;
	pixelY = 0;
	locationFound = false;
}


//
//  map_location
//
//  Sets state of object based on values of incoming strings representing
//  latitude and longitude.
//

void neattools_mapper::map_location( char* longitude, char* latitude ) {
	strcpy( lastLongitude, longitude );
	strcpy( lastLatitude, latitude );
	int int_longitude = GPS_2_int( longitude );
	int int_latitude = GPS_2_int( latitude );
	handle_new_location( int_longitude, int_latitude );
}

void neattools_mapper::d_map_location( double longitude, double latitude ) {
	int int_longitude = (int)(longitude * 100000);
	int int_latitude = (int)(latitude * 100000);
	handle_new_location( int_longitude, int_latitude );
}

void neattools_mapper::handle_new_location( int int_longitude, int int_latitude ) {
	lastIntLongitude = int_longitude;
	lastIntLatitude = int_latitude;
	if ( gpsMap.isActive( int_longitude, int_latitude )) {
		pixelX = gpsMap.longitude_to_xPixel( int_longitude );
		pixelY = gpsMap.latitude_to_yPixel( int_latitude );
		locationFound = true;
	} else {
		locationFound = false;
	}
}


char* neattools_mapper::toString() {
	static char buf[200];
	sprintf( buf, "neattools_mapper::toString, locationFound %d, %d, %d, last Long/Lat %s/%s as int: %d, %d\n", locationFound, pixelX, pixelY, lastLongitude, lastLatitude, lastIntLongitude, lastIntLatitude );
	return( &buf[0] );
}

char* neattools_mapper::toString2() {
	return( gpsMap.toString() );
}

